@@ -32,6 +32,10 @@
 #include <algorithm>
 #include "threadstatesaver.h"
 
+namespace {
+    void _destroyParentInfo(SbkObject* obj, bool keepReference);
+}
+
 extern "C"
 {
 
@@ -39,8 +43,7 @@ static void SbkObjectTypeDealloc(PyObject* pyObj);
 static PyObject* SbkObjectTypeTpNew(PyTypeObject* metatype, PyObject* args, PyObject* kwds);
 
 PyTypeObject SbkObjectType_Type = {
-    PyObject_HEAD_INIT(0)
-    /*ob_size*/             0,
+    PyVarObject_HEAD_INIT(0, 0)
     /*tp_name*/             "Shiboken.ObjectType",
     /*tp_basicsize*/        sizeof(SbkObjectType),
     /*tp_itemsize*/         0,
@@ -105,6 +108,26 @@ static PyGetSetDef SbkObjectGetSetList[] = {
 static int SbkObject_traverse(PyObject* self, visitproc visit, void* arg)
 {
     SbkObject* sbkSelf = reinterpret_cast<SbkObject*>(self);
+
+    //Visit children
+    Shiboken::ParentInfo* pInfo = sbkSelf->d->parentInfo;
+    if (pInfo) {
+        std::set<SbkObject*>::const_iterator it = pInfo->children.begin();
+        for(; it != pInfo->children.end(); ++it)
+            Py_VISIT(*it);
+    }
+
+    //Visit refs
+    Shiboken::RefCountMap* rInfo = sbkSelf->d->referredObjects;
+    if (rInfo) {
+        Shiboken::RefCountMap::const_iterator it = rInfo->begin();
+        for (; it != rInfo->end(); ++it) {
+            std::list<PyObject*>::const_iterator ref = it->second.begin();
+            for(; ref != it->second.end(); ++ref)
+                Py_VISIT(*ref);
+        }
+    }
+
     if (sbkSelf->ob_dict)
         Py_VISIT(sbkSelf->ob_dict);
     return 0;
@@ -113,14 +136,21 @@ static int SbkObject_traverse(PyObject* self, visitproc visit, void* arg)
 static int SbkObject_clear(PyObject* self)
 {
     SbkObject* sbkSelf = reinterpret_cast<SbkObject*>(self);
+
+    Shiboken::Object::removeParent(sbkSelf);
+
+    if (sbkSelf->d->parentInfo)
+        _destroyParentInfo(sbkSelf, true);
+
+    Shiboken::Object::clearReferences(sbkSelf);
+
     if (sbkSelf->ob_dict)
         Py_CLEAR(sbkSelf->ob_dict);
     return 0;
 }
 
 SbkObjectType SbkObject_Type = { { {
-    PyObject_HEAD_INIT(&SbkObjectType_Type)
-    /*ob_size*/             0,
+    PyVarObject_HEAD_INIT(&SbkObjectType_Type, 0)
     /*tp_name*/             "Shiboken.Object",
     /*tp_basicsize*/        sizeof(SbkObject),
     /*tp_itemsize*/         0,
@@ -187,7 +217,8 @@ void SbkDeallocWrapper(PyObject* pyObj)
             Shiboken::Object::deallocData(sbkObj, true);
 
             Shiboken::ThreadStateSaver threadSaver;
-            threadSaver.save();
+            if (Py_IsInitialized())
+                threadSaver.save();
             sbkType->d->cpp_dtor(cptr);
         }
     } else {
@@ -225,6 +256,7 @@ void SbkObjectTypeDealloc(PyObject* pyObj)
 
 PyObject* SbkObjectTypeTpNew(PyTypeObject* metatype, PyObject* args, PyObject* kwds)
 {
+#ifndef IS_PY3K
     // Check if all bases are new style before calling type.tp_new
     // Was causing gc assert errors in test_bug704.py when
     // this check happened after creating the type object.
@@ -247,6 +279,7 @@ PyObject* SbkObjectTypeTpNew(PyTypeObject* metatype, PyObject* args, PyObject* k
             return 0;
         }
     }
+#endif
 
     // The meta type creates a new type when the Python programmer extends a wrapped C++ class.
     SbkObjectType* newType = reinterpret_cast<SbkObjectType*>(PyType_Type.tp_new(metatype, args, kwds));
@@ -320,6 +353,26 @@ PyObject* SbkObjectTpNew(PyTypeObject* subtype, PyObject*, PyObject*)
 
 } //extern "C"
 
+
+namespace
+{
+
+void _destroyParentInfo(SbkObject* obj, bool keepReference)
+{
+    Shiboken::ParentInfo* pInfo = obj->d->parentInfo;
+    if (pInfo) {
+        while(!pInfo->children.empty()) {
+            SbkObject* first = *pInfo->children.begin();
+            // Mark child as invalid
+            Shiboken::Object::invalidate(first);
+            Shiboken::Object::removeParent(first, false, keepReference);
+        }
+        Shiboken::Object::removeParent(obj, false);
+    }
+}
+
+}
+
 namespace Shiboken
 {
 
@@ -371,9 +424,14 @@ bool importModule(const char* moduleName, PyTypeObject*** cppApiPtr)
     if (cppApi.isNull())
         return false;
 
+#ifdef IS_PY3K
+    if (PyCapsule_CheckExact(cppApi))
+        *cppApiPtr = reinterpret_cast<PyTypeObject**>(PyCapsule_GetPointer(cppApi, 0));
+#else
+    // Python 2.6 doesn't have PyCapsule API, so let's keep usign PyCObject on all Python 2.x
     if (PyCObject_Check(cppApi))
         *cppApiPtr = reinterpret_cast<PyTypeObject**>(PyCObject_AsVoidPtr(cppApi));
-
+#endif
     return true;
 }
 
@@ -434,10 +492,7 @@ void setErrorAboutWrongArguments(PyObject* args, const char* funcName, const cha
                 if (i)
                     params += ", ";
                 PyObject* arg = PyTuple_GET_ITEM(args, i);
-                if (PyCObject_Check(arg))
-                    params += "pointer";
-                else
-                    params += arg->ob_type->tp_name;
+                params += arg->ob_type->tp_name;
             }
         } else {
             params = args->ob_type->tp_name;
@@ -587,14 +642,21 @@ const char* getOriginalName(SbkObjectType* self)
     return self->d->original_name;
 }
 
-void setTypeDiscoveryFunction(SbkObjectType* self, TypeDiscoveryFunc func)
+void setTypeDiscoveryFunctionV2(SbkObjectType* self, TypeDiscoveryFuncV2 func)
 {
     self->d->type_discovery = func;
 }
 
+void setTypeDiscoveryFunction(SbkObjectType* self, TypeDiscoveryFunc func)
+{
+    self->d->type_discovery = (TypeDiscoveryFuncV2)func;
+}
+
 TypeDiscoveryFunc getTypeDiscoveryFunction(SbkObjectType* self)
 {
-    return self->d->type_discovery;
+    // This is an illegal cast because the return value is different,
+    // but nobody ever used this function, so... =]
+    return (TypeDiscoveryFunc)self->d->type_discovery;
 }
 
 void copyMultimpleheritance(SbkObjectType* self, SbkObjectType* other)
@@ -675,6 +737,7 @@ void setTypeUserData(SbkObjectType* self, void* userData, DeleteUserDataFunc d_f
 
 } // namespace ObjectType
 
+
 namespace Object
 {
 
@@ -707,22 +770,6 @@ static void setSequenceOwnership(PyObject* pyObj, bool owner)
     }
 }
 
-
-static void _destroyParentInfo(SbkObject* obj, bool keepReference)
-{
-    ParentInfo* pInfo = obj->d->parentInfo;
-    if (pInfo) {
-        while(!pInfo->children.empty()) {
-            SbkObject* first = *pInfo->children.begin();
-            // Mark child as invalid
-            Shiboken::Object::invalidate(first);
-            removeParent(first, false, keepReference);
-       }
-       removeParent(obj, false);
-    }
-}
-
-
 void setValidCpp(SbkObject* pyObj, bool value)
 {
     pyObj->d->validCppObject = value;
@@ -771,7 +818,7 @@ void getOwnership(PyObject* pyObj)
 void releaseOwnership(SbkObject* self)
 {
     // skip if the ownership have already moved to c++
-    SbkObjectType* selfType = reinterpret_cast<SbkObjectType*>(self->ob_type);
+    SbkObjectType* selfType = reinterpret_cast<SbkObjectType*>(Py_TYPE(self));
     if (!self->d->hasOwnership || selfType->d->type_behaviour == BEHAVIOUR_VALUETYPE)
         return;
 
@@ -824,6 +871,20 @@ void invalidate(SbkObject* self)
                 removeParent(*it, true, true);
         }
     }
+
+    // If has ref to other objects invalidate all
+    if (self->d->referredObjects) {
+        RefCountMap& refCountMap = *(self->d->referredObjects);
+        RefCountMap::iterator iter;
+        for (iter = refCountMap.begin(); iter != refCountMap.end(); ++iter) {
+            const std::list<PyObject*> lst = iter->second;
+            std::list<PyObject*>::const_iterator it = lst.begin();
+            while(it != lst.end()) {
+                invalidate(*it);
+                ++it;
+            }
+        }
+    }
 }
 
 void makeValid(SbkObject* self)
@@ -841,6 +902,21 @@ void makeValid(SbkObject* self)
         for (; it != self->d->parentInfo->children.end(); ++it)
             makeValid(*it);
     }
+
+    // If has ref to other objects make all valid again
+    if (self->d->referredObjects) {
+        RefCountMap& refCountMap = *(self->d->referredObjects);
+        RefCountMap::iterator iter;
+        for (iter = refCountMap.begin(); iter != refCountMap.end(); ++iter) {
+            const std::list<PyObject*> lst = iter->second;
+            std::list<PyObject*>::const_iterator it = lst.begin();
+            while(it != lst.end()) {
+                if (Shiboken::Object::checkType(*it))
+                    makeValid(reinterpret_cast<SbkObject*>(*it));
+                ++it;
+            }
+        }
+    }
 }
 
 bool hasParentInfo(SbkObject* pyObj)
@@ -850,7 +926,7 @@ bool hasParentInfo(SbkObject* pyObj)
 
 void* cppPointer(SbkObject* pyObj, PyTypeObject* desiredType)
 {
-    PyTypeObject* type = pyObj->ob_type;
+    PyTypeObject* type = Py_TYPE(pyObj);
     int idx = 0;
     if (reinterpret_cast<SbkObjectType*>(type)->d->is_multicpp)
         idx = getTypeIndexOnHierarchy(type, desiredType);
@@ -862,8 +938,8 @@ void* cppPointer(SbkObject* pyObj, PyTypeObject* desiredType)
 bool setCppPointer(SbkObject* sbkObj, PyTypeObject* desiredType, void* cptr)
 {
     int idx = 0;
-    if (reinterpret_cast<SbkObjectType*>(sbkObj->ob_type)->d->is_multicpp)
-        idx = getTypeIndexOnHierarchy(sbkObj->ob_type, desiredType);
+    if (reinterpret_cast<SbkObjectType*>(Py_TYPE(sbkObj))->d->is_multicpp)
+        idx = getTypeIndexOnHierarchy(Py_TYPE(sbkObj), desiredType);
 
     bool alreadyInitialized = sbkObj->d->cptr[idx];
     if (alreadyInitialized)
@@ -878,7 +954,7 @@ bool setCppPointer(SbkObject* sbkObj, PyTypeObject* desiredType, void* cptr)
 bool isValid(PyObject* pyObj)
 {
     if (!pyObj || pyObj == Py_None
-        || pyObj->ob_type->ob_type != &SbkObjectType_Type) {
+        || Py_TYPE(pyObj->ob_type) != &SbkObjectType_Type) {
         return true;
     }
 
@@ -905,13 +981,13 @@ bool isValid(SbkObject* pyObj, bool throwPyError)
     SbkObjectPrivate* priv = pyObj->d;
     if (!priv->cppObjectCreated && isUserType(reinterpret_cast<PyObject*>(pyObj))) {
         if (throwPyError)
-            PyErr_Format(PyExc_RuntimeError, "Base constructor of the object (%s) not called.", pyObj->ob_type->tp_name);
+            PyErr_Format(PyExc_RuntimeError, "Base constructor of the object (%s) not called.", Py_TYPE(pyObj)->tp_name);
         return false;
     }
 
     if (!priv->validCppObject) {
         if (throwPyError)
-            PyErr_Format(PyExc_RuntimeError, "Internal C++ object (%s) already deleted.", pyObj->ob_type->tp_name);
+            PyErr_Format(PyExc_RuntimeError, "Internal C++ object (%s) already deleted.", Py_TYPE(pyObj)->tp_name);
         return false;
     }
 
@@ -942,7 +1018,7 @@ PyObject* newObject(SbkObjectType* instanceType,
                 instanceType = reinterpret_cast<SbkObjectType*>(tr->pythonType());
         }
         if (!tr)
-            instanceType = BindingManager::instance().resolveType(cptr, instanceType);
+            instanceType = BindingManager::instance().resolveType(&cptr, instanceType);
     }
 
     SbkObject* self = reinterpret_cast<SbkObject*>(SbkObjectTpNew(reinterpret_cast<PyTypeObject*>(instanceType), 0, 0));
@@ -1008,11 +1084,10 @@ void removeParent(SbkObject* child, bool giveOwnershipBack, bool keepReference)
     if (!pInfo || !pInfo->parent) {
         if (pInfo && pInfo->hasWrapperRef) {
             pInfo->hasWrapperRef = false;
-            delete pInfo;
-            child->d->parentInfo = 0;
         }
         return;
     }
+
     ChildrenList& oldBrothers = pInfo->parent->d->parentInfo->children;
     // Verify if this child is part of parent list
     ChildrenList::iterator iChild = std::find(oldBrothers.begin(), oldBrothers.end(), child);
@@ -1028,7 +1103,7 @@ void removeParent(SbkObject* child, bool giveOwnershipBack, bool keepReference)
         child->d->containsCppWrapper) {
         //If have already a extra ref remove this one
         if (pInfo->hasWrapperRef)
-            Py_CLEAR(child);
+            Py_DECREF(child);
         else
             pInfo->hasWrapperRef = true;
         return;
@@ -1037,14 +1112,8 @@ void removeParent(SbkObject* child, bool giveOwnershipBack, bool keepReference)
     // Transfer ownership back to Python
     child->d->hasOwnership = giveOwnershipBack;
 
-    if (pInfo->children.empty()) {
-        // Erase parentInfo data
-        delete pInfo;
-        child->d->parentInfo = 0;
-    }
-
     // Remove parent ref
-    Py_CLEAR(child);
+    Py_DECREF(child);
 }
 
 void setParent(PyObject* parent, PyObject* child)
@@ -1135,7 +1204,7 @@ void deallocData(SbkObject* self, bool cleanup)
 
 void setTypeUserData(SbkObject* wrapper, void* userData, DeleteUserDataFunc d_func)
 {
-    SbkObjectType* ob_type = reinterpret_cast<SbkObjectType*>(wrapper->ob_type);
+    SbkObjectType* ob_type = reinterpret_cast<SbkObjectType*>(Py_TYPE(wrapper));
     if (ob_type->d->user_data)
         ob_type->d->d_func(ob_type->d->user_data);
 
@@ -1145,7 +1214,7 @@ void setTypeUserData(SbkObject* wrapper, void* userData, DeleteUserDataFunc d_fu
 
 void* getTypeUserData(SbkObject* wrapper)
 {
-    return reinterpret_cast<SbkObjectType*>(wrapper->ob_type)->d->user_data;
+    return reinterpret_cast<SbkObjectType*>(Py_TYPE(wrapper))->d->user_data;
 }
 
 void keepReference(SbkObject* self, const char* key, PyObject* referredObject, bool append)
@@ -1210,8 +1279,7 @@ void clearReferences(SbkObject* self)
     RefCountMap::iterator iter;
     for (iter = refCountMap.begin(); iter != refCountMap.end(); ++iter)
         decRefPyObjectList(iter->second);
-    delete self->d->referredObjects;
-    self->d->referredObjects = 0;
+    self->d->referredObjects->clear();
 }
 
 } // namespace Object
