@@ -23,7 +23,7 @@
 #ifndef BASEWRAPPER_P_H
 #define BASEWRAPPER_P_H
 
-#include <Python.h>
+#include "sbkpython.h"
 #include <list>
 #include <map>
 
@@ -80,6 +80,14 @@ struct SbkObjectPrivate
     Shiboken::ParentInfo* parentInfo;
     /// Manage reference counting of objects that are referred but not owned.
     Shiboken::RefCountMap* referredObjects;
+
+    ~SbkObjectPrivate()
+    {
+        delete parentInfo;
+        parentInfo = 0;
+        delete referredObjects;
+        referredObjects = 0;
+    }
 };
 
 /// The type behaviour was not defined yet
@@ -96,7 +104,7 @@ struct SbkObjectTypePrivate
 
     /// Special cast function, null if this class doesn't have multiple inheritance.
     SpecialCastFunction mi_specialcast;
-    TypeDiscoveryFunc type_discovery;
+    TypeDiscoveryFuncV2 type_discovery;
     /// Extended "isConvertible" function to be used when a conversion operator is defined in another module.
     ExtendedIsConvertibleFunc ext_isconvertible;
     /// Extended "toCpp" function to be used when a conversion operator is defined in another module.
