@@ -27,16 +27,24 @@ namespace Shiboken
 
 GilState::GilState()
 {
-    if(Py_IsInitialized())
+    if(Py_IsInitialized()) {
         m_gstate = PyGILState_Ensure();
+        m_locked = true;
+    }
 }
 
 GilState::~GilState()
 {
-    if(Py_IsInitialized())
-        PyGILState_Release(m_gstate);
+    release();
 }
 
+void GilState::release()
+{
+    if(m_locked && Py_IsInitialized()) {
+        PyGILState_Release(m_gstate);
+        m_locked = false;
+    }
+}
 } // namespace Shiboken
 
 
