@@ -21,21 +21,26 @@
  */
 
 #include "sbkenum.h"
+#include "sbkstring.h"
 #include "basewrapper.h"
-#include <string.h>
-#include <cstring>
-#include <list>
 #include "sbkdbg.h"
 #include "autodecref.h"
 #include "typeresolver.h"
 
+#include <string.h>
+#include <cstring>
+#include <list>
+
+
+#define SBK_ENUM(ENUM)  reinterpret_cast<SbkEnumObject*>(ENUM)
+
 extern "C"
 {
 
 struct SbkEnumObject
 {
     PyObject_HEAD
-    long ob_ival;
+    long ob_value;
     PyObject* ob_name;
 };
 
@@ -43,9 +48,9 @@ static PyObject* SbkEnumObject_repr(PyObject* self)
 {
     PyObject* enumName = ((SbkEnumObject*)self)->ob_name;
     if (enumName)
-        return PyString_FromFormat("%s.%s", self->ob_type->tp_name, PyString_AS_STRING(enumName));
+        return Shiboken::String::fromFormat("%s.%s", self->ob_type->tp_name, PyBytes_AS_STRING(enumName));
     else
-        return PyString_FromFormat("%s(%ld)", self->ob_type->tp_name, ((SbkEnumObject*)self)->ob_ival);
+        return Shiboken::String::fromFormat("%s(%ld)", self->ob_type->tp_name, ((SbkEnumObject*)self)->ob_value);
 }
 
 static int SbkEnumObject_print(PyObject* self, FILE* fp, int)
@@ -53,9 +58,9 @@ static int SbkEnumObject_print(PyObject* self, FILE* fp, int)
     Py_BEGIN_ALLOW_THREADS
     PyObject* enumName = ((SbkEnumObject*)self)->ob_name;
     if (enumName)
-        fprintf(fp, "%s.%s", self->ob_type->tp_name, PyString_AS_STRING(enumName));
+        fprintf(fp, "%s.%s", self->ob_type->tp_name, PyBytes_AS_STRING(enumName));
     else
-        fprintf(fp, "%s(%ld)", self->ob_type->tp_name, ((SbkEnumObject*)self)->ob_ival);
+        fprintf(fp, "%s(%ld)", self->ob_type->tp_name, ((SbkEnumObject*)self)->ob_value);
     Py_END_ALLOW_THREADS
     return 0;
 }
@@ -68,14 +73,14 @@ static PyObject* SbkEnumObject_name(PyObject* self, void*)
 
 static PyObject* SbkEnum_tp_new(PyTypeObject* type, PyObject* args, PyObject* kwds)
 {
-    int itemValue = 0;
-    if (!PyArg_ParseTuple(args, "|i:__new__", &itemValue))
+    long itemValue = 0;
+    if (!PyArg_ParseTuple(args, "|l:__new__", &itemValue))
         return 0;
 
     SbkEnumObject* self = PyObject_New(SbkEnumObject, type);
     if (!self)
         return 0;
-    self->ob_ival = itemValue;
+    self->ob_value = itemValue;
     PyObject* item = Shiboken::Enum::getEnumItemFromValue(type, itemValue);
     if (item) {
         self->ob_name = SbkEnumObject_name(item, 0);
@@ -86,14 +91,204 @@ static PyObject* SbkEnum_tp_new(PyTypeObject* type, PyObject* args, PyObject* kw
     return reinterpret_cast<PyObject*>(self);
 }
 
+static PyObject* enum_int(PyObject* v)
+{
+#ifdef IS_PY3K
+    return PyLong_FromLong(SBK_ENUM(v)->ob_value);
+#else
+    return PyInt_FromLong(SBK_ENUM(v)->ob_value);
+#endif
+}
+
+static long getNumberValue(PyObject* v)
+{
+    PyObject* number = PyNumber_Long(v);
+    long result = PyLong_AsLong(number);
+    Py_XDECREF(number);
+    return result;
+}
+
+
+static PyObject* enum_and(PyObject* self, PyObject* b)
+{
+    if (!PyNumber_Check(b)) {
+        Py_INCREF(Py_NotImplemented);
+        return Py_NotImplemented;
+    }
+
+    long valA = SBK_ENUM(self)->ob_value;
+    long valB = getNumberValue(b);
+    return PyLong_FromLong(valA & valB);
+}
+
+static PyObject* enum_or(PyObject* self, PyObject* b)
+{
+    if (!PyNumber_Check(b)) {
+        Py_INCREF(Py_NotImplemented);
+        return Py_NotImplemented;
+    }
+
+    long valA = SBK_ENUM(self)->ob_value;
+    long valB = getNumberValue(b);
+    return PyLong_FromLong(valA | valB);
+}
+
+static PyObject* enum_xor(PyObject* self, PyObject* b)
+{
+    if (!PyNumber_Check(b)) {
+        Py_INCREF(Py_NotImplemented);
+        return Py_NotImplemented;
+    }
+
+    long valA = SBK_ENUM(self)->ob_value;
+    long valB = getNumberValue(b);
+    return PyLong_FromLong(valA ^ valB);
+}
+
+
+static int enum_bool(PyObject* v)
+{
+    return (SBK_ENUM(v)->ob_value > 0);
+}
+
+static PyObject* enum_add(PyObject* self, PyObject* v)
+{
+    long valA = SBK_ENUM(self)->ob_value;
+    long valB = getNumberValue(v);
+    return PyLong_FromLong(valA + valB);
+}
+
+static PyObject* enum_subtract(PyObject* self, PyObject* v)
+{
+    long valA = SBK_ENUM(self)->ob_value;
+    long valB = getNumberValue(v);
+    return PyLong_FromLong(valA - valB);
+}
+
+static PyObject* enum_multiply(PyObject* self, PyObject* v)
+{
+    long valA = SBK_ENUM(self)->ob_value;
+    long valB = getNumberValue(v);
+    return PyLong_FromLong(valA * valB);
+}
+
+#ifndef IS_PY3K
+static PyObject* enum_divide(PyObject* self, PyObject* v)
+{
+    long valA = SBK_ENUM(self)->ob_value;
+    long valB = getNumberValue(v);
+    return PyLong_FromLong(valA / valB);
+}
+#endif
+
+static PyObject* enum_richcompare(PyObject* self, PyObject* other, int op)
+{
+    int result = 0;
+    if (!PyNumber_Check(other)) {
+        Py_INCREF(Py_NotImplemented);
+        return Py_NotImplemented;
+    }
+
+    long valA = SBK_ENUM(self)->ob_value;
+    long valB = getNumberValue(other);
+
+    switch (op) {
+    case Py_EQ:
+        result = (valA == valB);
+        break;
+    case Py_NE:
+        result = (valA != valB);
+        break;
+    case Py_LE:
+        result = (valA <= valB);
+        break;
+    case Py_GE:
+        result = (valA >= valB);
+        break;
+    case Py_LT:
+        result = (valA < valB);
+        break;
+    case Py_GT:
+        result = (valA > valB);
+        break;
+    default:
+        PyErr_BadArgument();
+        return NULL;
+    }
+    if (result)
+        Py_RETURN_TRUE;
+    else
+        Py_RETURN_FALSE;
+}
+
+static Py_hash_t enum_hash(PyObject* pyObj)
+{
+    return PyObject_Hash(reinterpret_cast<SbkEnumObject*>(pyObj)->ob_name);
+}
+
 static PyGetSetDef SbkEnumGetSetList[] = {
     {const_cast<char*>("name"), &SbkEnumObject_name},
     {0}  // Sentinel
 };
 
+static PyNumberMethods enum_as_number = {
+     /* nb_add */                   enum_add,
+     /* nb_subtract */              enum_subtract,
+     /* nb_multiply */              enum_multiply,
+#ifndef IS_PY3K
+     /* nb_divide */                enum_divide,
+#endif
+     /* nb_remainder */             0,
+     /* nb_divmod */                0,
+     /* nb_power */                 0,
+     /* nb_negative */              0,
+     /* nb_positive */              enum_int,
+     /* nb_absolute */              0,
+     /* nb_bool/nb_nonzero */       enum_bool,
+     /* nb_invert */                0,
+     /* nb_lshift */                0,
+     /* nb_rshift */                0,
+     /* nb_and */                   enum_and,
+     /* nb_xor */                   enum_xor,
+     /* nb_or */                    enum_or,
+#ifndef IS_PY3K
+     /* nb_coerce */                0,
+#endif
+     /* nb_int */                   enum_int,
+#ifdef IS_PY3K
+     /* nb_reserved */              0,
+     /* nb_float */                 0,
+#else
+     /* nb_long */                  enum_int,
+     /* nb_float */                 0,
+     /* nb_oct */                   0,
+     /* nb_hex */                   0,
+#endif
+
+     /* nb_inplace_add */           0,
+     /* nb_inplace_subtract */      0,
+     /* nb_inplace_multiply */      0,
+#ifndef IS_PY3K
+     /* nb_inplace_div */           0,
+#endif
+     /* nb_inplace_remainder */     0,
+     /* nb_inplace_power */         0,
+     /* nb_inplace_lshift */        0,
+     /* nb_inplace_rshift */        0,
+     /* nb_inplace_and */           0,
+     /* nb_inplace_xor */           0,
+     /* nb_inplace_or */            0,
+
+     /* nb_floor_divide */          0,
+     /* nb_true_divide */           0,
+     /* nb_inplace_floor_divide */  0,
+     /* nb_inplace_true_divide */   0,
+
+     /* nb_index */                 enum_int
+};
+
 PyTypeObject SbkEnumType_Type = {
-    PyObject_HEAD_INIT(0)
-    /*ob_size*/             0,
+    PyVarObject_HEAD_INIT(0, 0)
     /*tp_name*/             "Shiboken.EnumType",
     /*tp_basicsize*/        sizeof(PyTypeObject),
     /*tp_itemsize*/         0,
@@ -103,7 +298,7 @@ PyTypeObject SbkEnumType_Type = {
     /*tp_setattr*/          0,
     /*tp_compare*/          0,
     /*tp_repr*/             0,
-    /*tp_as_number*/        0,
+    /*tp_as_number*/        &enum_as_number,
     /*tp_as_sequence*/      0,
     /*tp_as_mapping*/       0,
     /*tp_hash*/             0,
@@ -112,7 +307,7 @@ PyTypeObject SbkEnumType_Type = {
     /*tp_getattro*/         0,
     /*tp_setattro*/         0,
     /*tp_as_buffer*/        0,
-    /*tp_flags*/            Py_TPFLAGS_DEFAULT|Py_TPFLAGS_BASETYPE,
+    /*tp_flags*/            Py_TPFLAGS_DEFAULT|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_CHECKTYPES,
     /*tp_doc*/              0,
     /*tp_traverse*/         0,
     /*tp_clear*/            0,
@@ -161,6 +356,11 @@ private:
 
 namespace Enum {
 
+bool check(PyObject* pyObj)
+{
+    return Py_TYPE(pyObj->ob_type) == &SbkEnumType_Type;
+}
+
 PyObject* getEnumItemFromValue(PyTypeObject* enumType, long itemValue)
 {
     PyObject *key, *value;
@@ -169,7 +369,7 @@ PyObject* getEnumItemFromValue(PyTypeObject* enumType, long itemValue)
 
     while (PyDict_Next(values, &pos, &key, &value)) {
         SbkEnumObject* obj = (SbkEnumObject*)value;
-        if (obj->ob_ival == itemValue) {
+        if (obj->ob_value == itemValue) {
             Py_INCREF(obj);
             return reinterpret_cast<PyObject*>(obj);
         }
@@ -264,8 +464,8 @@ PyObject* newItem(PyTypeObject* enumType, long itemValue, const char* itemName)
     if (!enumObj)
         return 0;
 
-    enumObj->ob_name = itemName ? PyString_FromString(itemName) : 0;
-    enumObj->ob_ival = itemValue;
+    enumObj->ob_name = itemName ? PyBytes_FromString(itemName) : 0;
+    enumObj->ob_value = itemValue;
 
     if (newValue) {
         PyObject* values = PyDict_GetItemString(enumType->tp_dict, const_cast<char*>("values"));
@@ -290,16 +490,18 @@ PyTypeObject* newTypeWithName(const char* name, const char* cppName)
 {
     PyTypeObject* type = new PyTypeObject;
     ::memset(type, 0, sizeof(PyTypeObject));
-    type->ob_type = &SbkEnumType_Type;
+    Py_TYPE(type) = &SbkEnumType_Type;
     type->tp_basicsize = sizeof(SbkEnumObject);
     type->tp_print = &SbkEnumObject_print;
     type->tp_repr = &SbkEnumObject_repr;
     type->tp_str = &SbkEnumObject_repr;
-    type->tp_flags = Py_TPFLAGS_DEFAULT;
-    type->tp_base = &PyInt_Type;
+    type->tp_flags = Py_TPFLAGS_DEFAULT|Py_TPFLAGS_CHECKTYPES;
     type->tp_name = name;
     type->tp_getset = SbkEnumGetSetList;
     type->tp_new = SbkEnum_tp_new;
+    type->tp_as_number = &enum_as_number;
+    type->tp_richcompare = &enum_richcompare;
+    type->tp_hash = &enum_hash;
 
     DeclaredEnumTypes::instance().addEnumType(type, cppName);
     return type;
@@ -312,7 +514,7 @@ const char* getCppName(PyTypeObject* enumType)
 
 long int getValue(PyObject* enumItem)
 {
-    return reinterpret_cast<SbkEnumObject*>(enumItem)->ob_ival;
+    return reinterpret_cast<SbkEnumObject*>(enumItem)->ob_value;
 }
 
 } // namespace Enum
