@@ -23,7 +23,7 @@
 #ifndef THREADSTATESAVER_H
 #define THREADSTATESAVER_H
 
-#include <Python.h>
+#include "sbkpython.h"
 #include <shibokenmacros.h>
 
 namespace Shiboken
