@@ -32,10 +32,29 @@ import unittest
 
 import sample
 from sample import SampleNamespace, ObjectType, Event
+from py3kcompat import IS_PY3K, b
+
+def createTempFile():
+    if IS_PY3K:
+        import tempfile
+        return tempfile.SpooledTemporaryFile(mode='rw')
+    else:
+        return os.tmpfile()
 
 class EnumTest(unittest.TestCase):
     '''Test case for Python representation of C++ enums.'''
 
+    def testEnumRepr(self):
+        enum = SampleNamespace.Option(1)
+        self.assertEqual(eval(repr(enum)), enum)
+
+        enum = SampleNamespace.Option(999)
+        self.assertEqual(eval(repr(enum)), enum)
+
+    def testHashability(self):
+        '''Enums should be hashable and different enums with different values should have different hashes'''
+        self.assertNotEqual(hash(SampleNamespace.TwoIn), hash(SampleNamespace.TwoOut))
+
     def testEnumValuesInsideEnum(self):
         '''Enum values should be accessible inside the enum as well as outside.'''
         for value_name in SampleNamespace.Option.values:
@@ -54,12 +73,12 @@ class EnumTest(unittest.TestCase):
     def testBuildingEnumWithDefaultValue(self):
         '''Enum constructor with default value'''
         enum = SampleNamespace.Option()
-        self.assertEqual(enum, SampleNamespace.None)
+        self.assertEqual(enum, SampleNamespace.None_)
 
     def testEnumConversionToAndFromPython(self):
         '''Conversion of enum objects from Python to C++ back again.'''
         enumout = SampleNamespace.enumInEnumOut(SampleNamespace.TwoIn)
-        self.assert_(enumout, SampleNamespace.TwoOut)
+        self.assertTrue(enumout, SampleNamespace.TwoOut)
         self.assertEqual(repr(enumout), repr(SampleNamespace.TwoOut))
 
     def testEnumConstructorWithTooManyParameters(self):
@@ -88,10 +107,14 @@ class EnumTest(unittest.TestCase):
 
     def testEnumTpPrintImplementation(self):
         '''Without SbkEnum.tp_print 'print' returns the enum represented as an int.'''
-        tmpfile = os.tmpfile()
-        sys.stdout = tmpfile
-        print Event.ANY_EVENT
-        sys.stdout = sys.__stdout__
+        tmpfile = createTempFile()
+        if IS_PY3K:
+            from py3k import printToFile
+            printToFile(tmpfile, Event.ANY_EVENT)
+        else:
+            sys.stdout = tmpfile
+            print(Event.ANY_EVENT)
+            sys.stdout = sys.__stdout__
         tmpfile.seek(0)
         text = tmpfile.read().strip()
         tmpfile.close()
@@ -106,7 +129,7 @@ class MyEvent(Event):
 class OutOfBoundsTest(unittest.TestCase):
     def testValue(self):
         e = MyEvent()
-        self.assertEqual(repr(e.eventType()), '<enum-item sample.Event.EventType.#out of bounds# (999)>')
+        self.assertEqual(repr(e.eventType()), 'sample.Event.EventType(999)')
 
 class EnumOverloadTest(unittest.TestCase):
     '''Test case for overloads involving enums'''
@@ -118,6 +141,12 @@ class EnumOverloadTest(unittest.TestCase):
         self.assertEqual(o.callWithEnum('', Event.ANY_EVENT, 9), 81)
         self.assertEqual(o.callWithEnum('', 9), 9)
 
+class EnumOperators(unittest.TestCase):
+    '''Test case for operations on enums'''
+
+    def testInequalitySameObject(self):
+        self.assertFalse(Event.ANY_EVENT != Event.ANY_EVENT)
+
 
 if __name__ == '__main__':
     unittest.main()
